<?php

class content_custom extends def_module {
    /**
     * Функция для формирования уменьшенного и оригинального изображения на поддомене
     *
     * Параметры:
     * @subdomain - субдомен на котором должны храниться изображения - не забудьте создать в панели управления сайтом
     * // @save_folder - субдомен на котором должны храниться изображения - не забудьте создать в панели управления сайтом
     * @elem_id - указывает id объекта из которого нужно вытянуть инфомрацию
     * @field_name - указывает поле которое содержит изображение
     * @width - ширина уменьшенной копии
     * @height - высота уменьшенной копии
     * @crop - нужно ли обрезать изображение
     * @cropside - указывает положение рамки обрезания
     * @isLogo - указывает необходимость наложения водяного знака на изображение
     * @quality - указывает степень сжатия (качество) изображения
     *
     * Возвращаемые значения:
     * - домен
     * - поддомен
     * - уменьшенное изображение/оригинальное изображение:
     * --- высота/ширина уменьшенного изображения
     * --- размер
     * --- расширение
     * --- название файла
     * --- путь к файлу относительно DOCUMENT_ROOT
     * --- путь к файлу относительно поддомена сайта
     *
     * Дополнительно:
     * - Создание поддомена через .htaccess - http://goo.gl/Cscx8
     * - Модернизация system makeThumbnailFull() - http://goo.gl/0kcJBs
     */
    function makeSubDomainImage($subdomain = false, $elem_id = false, $field_name = false,
                                $width = 'auto', $height = 'auto', $crop = true,
                                $cropside = 5, $isLogo = false, $quality = 100) {
        $arr = Array();

        if(!$subdomain) {
            $arr['error'] = '<![CDATA[ Укажите поддомен($subdomain) ]]>';

            return $arr;
        }

        if($elem_id) {
            $element = umiHierarchy::getInstance()->getElement($elem_id);

            if($element instanceOf umiHierarchyElement) {
                if($field_name) {

                } else {
                    $arr['error'] = '<![CDATA[ Укажите название поля, в котором содержиться изображение($field_name) ]]>';

                    return $arr;
                }

                $image_elem = $element->getValue($field_name);
                $alt_name = $element->getAltName();

                if(is_object($image_elem)) {
                    $path = $image_elem->getFilepath();
                }
            }
        } else {
            $arr['error'] = '<![CDATA[ Не задан идентификатор объекта($elem_id) ]]>';

            return $arr;
        }

        $domain = getServer('HTTP_HOST');
        $thumbs_path = './' . $subdomain . '/thumbs/';
        $origin_path = './' . $subdomain . '/origin/';

        $isSharpen=true;

        $image = new umiImageFile($path);

        $file_name = $image->getFileName();
        $file_ext = strtolower($image->getExt());
        $file_ext = ($file_ext=='bmp'?'jpg':$file_ext);

        $allowedExts = Array('gif', 'jpeg', 'jpg', 'png', 'bmp');

        if(!in_array($file_ext, $allowedExts)) {
            $arr['error'] = '<![CDATA[ Не разрешенное расширение файла ]]>';

            return $arr;
        }

        // Если задан alt_name то нужно переименовать изображение
        if(iconv_strlen($alt_name) > 0) $file_name = $alt_name;
        else $file_name = substr($file_name, 0, (strlen($file_name) - (strlen($file_ext) + 1)) );

        /*Название папки в виде ширина_высота_вырезание_качество*/
        $thumbPath = $width . '_' . $height . '_' . $cropside . '_' . $quality;

        /*Определяет, является ли имя файла директорией*/
        if (!is_dir($thumbs_path . $thumbPath)) {
            /*Создаёт директорию*/
            mkdir($thumbs_path . $thumbPath, 0755, true);
        }

        /*Название уменьшенного изображения*/
        $file_name_new = $file_name . "." . $file_ext;

        /*Путь к уменьшенному изображению*/
        $path_new = $thumbs_path . $thumbPath . '/' . $file_name_new;

        if(!file_exists($path_new) || filemtime($path_new) < filemtime($path)) {
            if(file_exists($path_new)) {
                unlink($path_new);
            }

            $width_src = $image->getWidth();
            $height_src = $image->getHeight();

            if(!($width_src && $height_src)) {
                throw new coreException(getLabel('error-image-corrupted', null, $path));
            }

            if($height == "auto") {
                $real_height = (int) round($height_src * ($width / $width_src));
                //change
                $height=$real_height;
                $real_width = (int) $width;
            } else {
                if($width == "auto") {
                    $real_width = (int) round($width_src * ($height / $height_src));
                    //change
                    $width=$real_width;
                } else {
                    $real_width = (int) $width;
                }

                $real_height = (int) $height;
            }

            $offset_h=0;
            $offset_w=0;

            /*realloc: devision by zero fix*/
            if (!intval($width) || !intval($height)) {
                $crop = false;
            }

            if ($crop){
                $width_ratio = $width_src/$width;
                $height_ratio = $height_src/$height;

                if ($width_ratio > $height_ratio){
                    $offset_w = round(($width_src-$width*$height_ratio)/2);
                    $width_src = round($width*$height_ratio);
                } elseif ($width_ratio < $height_ratio){
                    $offset_h = round(($height_src-$height*$width_ratio)/2);
                    $height_src = round($height*$width_ratio);
                }


                if($cropside) {
                    switch ($cropside):
                        case 1:
                            $offset_w = 0;
                            $offset_h = 0;
                            break;
                        case 2:
                            $offset_h = 0;
                            break;
                        case 3:
                            $offset_w += $offset_w;
                            $offset_h = 0;
                            break;
                        case 4:
                            $offset_w = 0;
                            break;
                        case 5:
                            break;
                        case 6:
                            $offset_w += $offset_w;
                            break;
                        case 7:
                            $offset_w = 0;
                            $offset_h += $offset_h;
                            break;
                        case 8:
                            $offset_h += $offset_h;
                            break;
                        case 9:
                            $offset_w += $offset_w;
                            $offset_h += $offset_h;
                            break;
                    endswitch;
                }
            }

            $thumb = imagecreatetruecolor($real_width, $real_height);

            $source_array = $image->createImage($path);
            $source = $source_array['im'];

            if ($width*4 < $width_src && $height*4 < $height_src) {
                $_TMP=array();
                $_TMP['width'] = round($width*4);
                $_TMP['height'] = round($height*4);

                $_TMP['image'] = imagecreatetruecolor($_TMP['width'], $_TMP['height']);

                if ($file_ext == 'gif') {
                    $_TMP['image_white'] = imagecolorallocate($_TMP['image'], 255, 255, 255);
                    imagefill($_TMP['image'], 0, 0, $_TMP['image_white']);
                    imagecolortransparent($_TMP['image'], $_TMP['image_white']);
                    imagealphablending($source, TRUE);
                    imagealphablending($_TMP['image'], TRUE);
                } else {
                    imagealphablending($_TMP['image'], false);
                    imagesavealpha($_TMP['image'], true);
                }
                imagecopyresampled($_TMP['image'], $source, 0, 0, $offset_w, $offset_h, $_TMP['width'], $_TMP['height'], $width_src, $height_src);

                imageDestroy($source);

                $source = $_TMP['image'];
                $width_src = $_TMP['width'];
                $height_src = $_TMP['height'];

                $offset_w = 0;
                $offset_h = 0;
                unset($_TMP);
            }

            if ($file_ext == 'gif') {
                $thumb_white_color = imagecolorallocate($thumb, 255, 255, 255);
                imagefill($thumb, 0, 0, $thumb_white_color);
                imagecolortransparent($thumb, $thumb_white_color);
                imagealphablending($source, TRUE);
                imagealphablending($thumb, TRUE);
            } else {
                imagealphablending($thumb, false);
                imagesavealpha($thumb, true);
            }

            imagecopyresampled($thumb, $source, 0, 0, $offset_w, $offset_h, $width, $height, $width_src, $height_src);

            if($isSharpen) $thumb = makeThumbnailFullUnsharpMask($thumb,80,.5,3);

            switch($file_ext) {
                case 'gif':
                    $res = imagegif($thumb, $path_new);
                    break;
                case 'png':
                    $res = imagepng($thumb, $path_new);
                    break;
                default:
                    $res = imagejpeg($thumb, $path_new, $quality);
            }
            if(!$res) {
                throw new coreException(getLabel('label-errors-16008'));
            }

            imageDestroy($source);
            imageDestroy($thumb);

            if($isLogo) {
                umiImageFile::addWatermark($path_new);
            }
        }


        /*Создание нового уменьшенного изображения на поддомене*/
        $value = new umiImageFile($path_new);

        /*Техническая информация*/
        $arr['info']['nodes:elem_id'][] = $elem_id;
        $arr['info']['nodes:field_name'][] = $field_name;
        $arr['info']['nodes:path'][] = $path;
        $arr['info']['nodes:alt_name'][] = $alt_name;
        $arr['info']['nodes:width'][] = $width;
        $arr['info']['nodes:height'][] = $height;
        $arr['info']['nodes:crop'][] = $crop;
        $arr['info']['nodes:cropside'][] = $cropside;
        $arr['info']['nodes:isLogo'][] = $isLogo;
        $arr['info']['nodes:quality'][] = $quality;
        $arr['info']['nodes:domain'][] = $domain;
        $arr['info']['nodes:subdomain'][] = $subdomain . '.' .$domain;

        /*Уменьшенное изображение*/
        $arr['thumb_info']['nodes:width'][] = $value->getWidth();
        $arr['thumb_info']['nodes:height'][] = $value->getHeight();
        $arr['thumb_info']['nodes:size'][] = $value->getSize();
        $arr['thumb_info']['nodes:ext'][] = $value->getExt();
        $arr['thumb_info']['nodes:filename'][] = $value->getFileName();
        $arr['thumb_info']['nodes:filepath'][] = $value->getFilePath();
        $thumb_src = 'http://' . $subdomain . '.' .$domain . substr($value->getFilePath(true), (iconv_strlen($subdomain) +1 ));
        $arr['thumb_info']['nodes:src'][] = $thumb_src;


        /*Если задан alt_name то нужно заново обрабатывать увеличенное изображение*/
        /*Если нет, то нужно работать со старым изображением*/
        if(iconv_strlen($alt_name) > 0) {
            $file_name = $alt_name;
            /*Путь к увеличенному изображению*/
            $path_new_origin = $origin_path . '/' . $file_name . "." . $file_ext;

            if(!file_exists($path_new_origin) || filemtime($path_new_origin) < filemtime($path)) {
                /*Если файл существует*/
                if(file_exists($path_new_origin)) {
                    /*Удалить файл*/
                    unlink($path_new_origin);
                }

                /*Определяет, является ли имя файла директорией*/
                if (!is_dir($origin_path)) {
                    /*Создаёт директорию*/
                    mkdir($origin_path, 0755, true);
                }

                switch($file_ext) {
                    case 'gif':
                        $res = imagecreatefromgif($path);
                        imagegif($res, $path_new_origin);
                        break;
                    case 'png':
                        $res = imagecreatefrompng($path);
                        imagepng($res, $path_new_origin);
                        break;
                    default:
                        $res = imagecreatefromjpeg($path);
                        imagejpeg($res, $path_new_origin);
                }
                if(!$res) {
                    throw new coreException(getLabel('label-errors-16008'));
                }
                imagedestroy($res);

            }

            /*Создание нового изображения на подоменне*/
            $value_origin = new umiImageFile($path_new_origin);

            /*Большое изображение*/
            $arr['origin_info']['nodes:width'][] = $value_origin->getWidth();
            $arr['origin_info']['nodes:height'][] = $value_origin->getHeight();
            $arr['origin_info']['nodes:size'][] = $value_origin->getSize();
            $arr['origin_info']['nodes:ext'][] = $value_origin->getExt();
            $arr['origin_info']['nodes:filename'][] = $value_origin->getFileName();
            $arr['origin_info']['nodes:filepath'][] = $value_origin->getFilePath();
            $origin_src = 'http://' . $subdomain . '.' .$domain . substr($value_origin->getFilePath(true), (iconv_strlen($subdomain) +1 ));
            $arr['origin_info']['nodes:src'][] = $origin_src;
        }

        if(cmsController::getInstance()->getCurrentMode() == "admin") {
            $arr['src'] = str_replace("&", "&amp;", $arr['src']);
        }

        return $arr;
    }

}

?>
